"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = __importDefault(require("crypto"));
const fs_1 = require("fs");
const assert_1 = require("../util/assert");
const http_1 = require("../util/http");
const nextPowerOfTwo_1 = require("../util/nextPowerOfTwo");
const v4_1 = require("../util/v4");
const createBoundRequestFunction = ({ region, accessId, secretKey, vaultName, }) => ({ method, subpath, headers, body, }) => {
    const host = `glacier.${region}.amazonaws.com`;
    const path = `/-/vaults/${vaultName}${subpath}`;
    const isoDateTime = v4_1.getISOTime();
    headers["x-amz-date"] = isoDateTime;
    headers["x-amz-glacier-version"] = `2012-06-01`;
    if (body) {
        headers["x-amz-content-sha256"] = body.SHA256;
    }
    headers["Authorization"] = v4_1.createAuthHeader({
        isoDateTime,
        method,
        host,
        path,
        headers,
        contentSHA256: body ? body.SHA256 : v4_1.EMPTY_BODY_SHA256,
        service: "glacier",
        region,
        accessKeyId: accessId,
        secretAccessKey: secretKey,
    });
    return http_1.http({
        url: `https://${host}${path}`,
        method,
        headers,
        body: body && body.data,
    });
};
exports.parseAWSS3GlacierOptions = (args) => {
    // Call .toString to ensure value is not null/undefined.
    const region = args.region.toString();
    const accessId = args.access.toString();
    const secretKey = args.secret.toString();
    const vaultName = args.vault.toString();
    return {
        region, accessId, secretKey, vaultName,
    };
};
const buildFinalTreeHash = (hashes) => {
    while (hashes.length > 1) {
        const newHashes = [];
        for (let i = 0; i < hashes.length - 1; i += 2) {
            newHashes.push(crypto_1.default.createHash("sha256")
                .update(Buffer.concat(hashes.slice(i, i + 2)))
                .digest());
        }
        if (hashes.length % 2 === 1) {
            newHashes.push(hashes[hashes.length - 1]);
        }
        hashes = newHashes;
    }
    return hashes[0];
};
const MiB = 1024 * 1024;
// This is difficult, as it needs to build the tree hash while reading the part,
// not collect hashes for the entire part first (which might result in heavy
// memory usage for large part sizes).
const calculateTreeAndLinearHashesOfPart = ({ path, start, end }) => __awaiter(this, void 0, void 0, function* () {
    const tree = [];
    const linear = crypto_1.default.createHash("sha256");
    const fd = yield fs_1.promises.open(path, "r");
    try {
        for (let chunkStart = start; chunkStart <= end; chunkStart += MiB) {
            // Calculate chunk range and length.
            const chunkEnd = Math.min(chunkStart + MiB - 1, end);
            const chunkSize = chunkEnd - chunkStart + 1;
            // Read chunk.
            const chunk = Buffer.allocUnsafe(chunkSize);
            const { bytesRead } = yield fd.read(chunk, 0, chunkSize, chunkStart);
            assert_1.assertTrue(bytesRead === chunkSize);
            // Build linear hash.
            linear.update(chunk);
            // Build tree hash.
            tree.push({
                level: 1,
                hash: crypto_1.default.createHash("sha256").update(chunk).digest(),
            });
            while (tree.length > 1 && tree[tree.length - 1].level === tree[tree.length - 2].level) {
                const right = assert_1.assertExists(tree.pop());
                const left = assert_1.assertExists(tree.pop());
                tree.push({
                    level: right.level + 1,
                    hash: crypto_1.default.createHash("sha256")
                        .update(Buffer.concat([left.hash, right.hash]))
                        .digest(),
                });
            }
        }
    }
    finally {
        yield fd.close();
    }
    // Finalise tree hash.
    while (tree.length > 1) {
        const right = assert_1.assertExists(tree.pop());
        const left = assert_1.assertExists(tree.pop());
        tree.push({
            // To silence type errors.
            level: -1,
            hash: crypto_1.default.createHash("sha256")
                .update(Buffer.concat([left.hash, right.hash]))
                .digest(),
        });
    }
    return {
        tree: tree[0].hash,
        linear: linear.digest(),
    };
});
exports.AWSS3Glacier = {
    // See https://docs.aws.amazon.com/amazonglacier/latest/dev/uploading-archive-mpu.html for limits.
    minParts: 1,
    maxParts: 10000,
    minPartSize: 1024 * 1024,
    maxPartSize: 1024 * 1024 * 1024 * 4,
    fromOptions(options) {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                requester: createBoundRequestFunction({
                    accessId: options.accessId,
                    secretKey: options.secretKey,
                    region: options.region,
                    vaultName: options.vaultName,
                }),
                vaultName: options.vaultName,
            };
        });
    },
    completeUpload(s, uploadId, fileSize, partHashes) {
        return __awaiter(this, void 0, void 0, function* () {
            yield s.requester({
                subpath: `/multipart-uploads/${uploadId}`,
                method: "POST",
                headers: {
                    "x-amz-sha256-tree-hash": buildFinalTreeHash(partHashes).toString("hex"),
                    "x-amz-archive-size": fileSize,
                },
            });
        });
    },
    idealPartSize(_, fileSize) {
        return __awaiter(this, void 0, void 0, function* () {
            return nextPowerOfTwo_1.nextPowerOfTwo(fileSize / this.maxParts);
        });
    },
    initiateNewUpload(s, fileName, partSize) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield s.requester({
                headers: {
                    "x-amz-part-size": partSize.toString(),
                    "x-amz-archive-description": fileName,
                },
                method: "POST",
                subpath: "/multipart-uploads",
            });
            return http_1.getLastHeaderValue(res, "x-amz-multipart-upload-id");
        });
    },
    uploadPart(s, uploadId, part) {
        return __awaiter(this, void 0, void 0, function* () {
            const { path, start, end } = part;
            const { linear, tree } = yield calculateTreeAndLinearHashesOfPart({ path, start, end });
            const res = yield s.requester({
                subpath: `/multipart-uploads/${uploadId}`,
                method: "PUT",
                headers: {
                    "Content-Length": end - start + 1,
                    "Content-Range": `bytes ${start}-${end}/*`,
                    "x-amz-sha256-tree-hash": tree.toString("hex"),
                },
                body: {
                    data: fs_1.createReadStream(path, { start, end }),
                    SHA256: linear.toString("hex"),
                },
            });
            const serverTreeHash = Buffer.from(http_1.getLastHeaderValue(res, "x-amz-sha256-tree-hash"), "hex");
            assert_1.assertTrue(tree.equals(serverTreeHash));
            return tree;
        });
    }
};
//# sourceMappingURL=AWSS3Glacier.js.map