"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = __importDefault(require("crypto"));
exports.leftPad = (str, width, char = "0") => {
    str = `${str}`;
    return char.repeat(Math.max(0, width - str.length)) + str;
};
exports.getISOTime = () => {
    let d = new Date();
    return [
        exports.leftPad(d.getUTCFullYear(), 4),
        exports.leftPad(d.getUTCMonth() + 1, 2),
        exports.leftPad(d.getUTCDate(), 2),
        `T`,
        exports.leftPad(d.getUTCHours(), 2),
        exports.leftPad(d.getUTCMinutes(), 2),
        exports.leftPad(d.getUTCSeconds(), 2),
        `Z`
    ].join("");
};
exports.percentEncodeBytes = (str, encodeSlashes = true) => {
    if (typeof str === "number") {
        return `${str}`;
    }
    const encoded = [];
    for (let i = 0; i < str.length; i++) {
        const char = str[i];
        if ((char >= "A" && char <= "Z")
            || (char >= "a" && char <= "z")
            || (char >= "0" && char <= "9")
            || char === "_"
            || char === "-"
            || char === "~"
            || char === ".") {
            encoded.push(char);
        }
        else if (char === "/") {
            encoded.push(encodeSlashes ? "%2F" : char);
        }
        else {
            encoded.push("%" + char.charCodeAt(0).toString(16).toLocaleUpperCase());
        }
    }
    return encoded.join("");
};
exports.hashSHA256 = (data) => crypto_1.default.createHash("sha256").update(data).digest();
exports.EMPTY_BODY_SHA256 = exports.hashSHA256("").toString("hex");
exports.hmac = ({ secret, data, }) => crypto_1.default.createHmac("sha256", secret).update(data).digest();
exports.generateCanonicalRequest = ({ method, host, path, queryParameters, headers, contentSHA256, }) => {
    const canonicalRequest = [
        method,
        exports.percentEncodeBytes(path, false),
    ];
    if (!queryParameters) {
        canonicalRequest.push("");
    }
    else {
        const paramsEncodedNames = [];
        const paramsEncodedMap = {};
        for (const [name, value] of Object.entries(queryParameters)) {
            const encodedName = exports.percentEncodeBytes(name);
            paramsEncodedMap[encodedName] = value;
            paramsEncodedNames.push(encodedName);
        }
        paramsEncodedNames.sort();
        const queryAWSEncoded = [];
        for (const name of paramsEncodedNames) {
            const value = paramsEncodedMap[name];
            if (value === true) {
                queryAWSEncoded.push(`${name}=`);
            }
            else if (value === false) {
                // skip
            }
            else {
                queryAWSEncoded.push(`${name}=${exports.percentEncodeBytes(value)}`);
            }
        }
        canonicalRequest.push(queryAWSEncoded.join("&"));
    }
    const headerNames = ["host"];
    const headersLCMap = { host };
    if (headers) {
        for (const [name, value] of Object.entries(headers)) {
            const lowerCaseName = name.toLocaleLowerCase();
            headerNames.push(lowerCaseName);
            headersLCMap[lowerCaseName] = value;
        }
        headerNames.sort();
    }
    const signedHeaders = headerNames.join(";");
    for (const name of headerNames) {
        canonicalRequest.push(`${name}:${headersLCMap[name]}`);
    }
    canonicalRequest.push("");
    canonicalRequest.push(signedHeaders);
    canonicalRequest.push(contentSHA256);
    return {
        hash: exports.hashSHA256(canonicalRequest.join("\n")).toString("hex"),
        headers: signedHeaders,
    };
};
exports.deriveSigningKey = ({ secretAccessKey, region, service, isoDate, }) => {
    const dateKey = exports.hmac({ secret: `AWS4${secretAccessKey}`, data: isoDate });
    const dateRegionKey = exports.hmac({ secret: dateKey, data: region });
    const dateRegionServiceKey = exports.hmac({ secret: dateRegionKey, data: service });
    return exports.hmac({ secret: dateRegionServiceKey, data: "aws4_request" });
};
exports.sign = ({ derivedKey, stringToSign }) => exports.hmac({ data: stringToSign, secret: derivedKey }).toString("hex");
exports.createAuthHeader = ({ isoDateTime, method, host, path, queryParameters, headers, contentSHA256, service, region, accessKeyId, secretAccessKey, }) => {
    const isoDate = isoDateTime.slice(0, 8);
    const { hash: canonicalRequest, headers: signedHeadersJoined } = exports.generateCanonicalRequest({
        method,
        host,
        path,
        queryParameters,
        headers,
        contentSHA256,
    });
    const stringToSign = [
        "AWS4-HMAC-SHA256",
        isoDateTime,
        [isoDate, region, service, "aws4_request"].join("/"),
        canonicalRequest,
    ].join("\n");
    const derivedKey = exports.deriveSigningKey({
        secretAccessKey: secretAccessKey,
        region: region,
        service: service,
        isoDate: isoDate,
    });
    const signature = exports.sign({ derivedKey, stringToSign });
    return `AWS4-HMAC-SHA256 Credential=${accessKeyId}/${isoDate}/${region}/${service}/aws4_request, SignedHeaders=${signedHeadersJoined}, Signature=${signature}`;
};
//# sourceMappingURL=v4.js.map